@@ -30,45 +30,74 @@
 
 
 static void
-check_button_changed (GtkWidget *button, CpuFreqPluginConfigure *configure)
+update_sensitivity (const CpuFreqPluginConfigure *configure)
 {
-  if (button == configure->display_icon)
-    cpuFreq->options->show_icon =
-      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+  const CpuFreqPluginOptions *options = cpuFreq->options;
 
-  else if (button == configure->display_freq)
-    cpuFreq->options->show_label_freq =
-      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
-
-  else if (button == configure->display_governor)
-    cpuFreq->options->show_label_governor =
-      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
-
-  else if (button == configure->keep_compact)
-    cpuFreq->options->keep_compact =
-      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
-
-  else if (button == configure->one_line)
-    cpuFreq->options->one_line =
-      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
-
-  if (!cpuFreq->options->show_label_freq && !cpuFreq->options->show_label_governor)
+  if (!options->show_label_freq && !options->show_label_governor)
   {
-    if (!cpuFreq->options->show_icon)
-    {
-      cpuFreq->options->show_icon = TRUE;
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (configure->display_icon), TRUE);
-    }
-
     gtk_widget_set_sensitive (configure->display_icon, FALSE);
+    gtk_widget_set_sensitive (configure->fontcolor_hbox, FALSE);
+    gtk_widget_set_sensitive (configure->fontname_hbox, FALSE);
   }
   else
   {
     gtk_widget_set_sensitive (configure->display_icon, TRUE);
+    gtk_widget_set_sensitive (configure->fontcolor_hbox, TRUE);
+    gtk_widget_set_sensitive (configure->fontname_hbox, TRUE);
   }
 
-  cpufreq_prepare_label (cpuFreq);
-  cpufreq_update_icon (cpuFreq);
+  gtk_widget_set_sensitive (configure->icon_color_freq, options->show_icon);
+}
+
+
+
+static void
+validate_configuration (const CpuFreqPluginConfigure *configure)
+{
+  CpuFreqPluginOptions *options = cpuFreq->options;
+
+  if (!options->show_label_freq && !options->show_label_governor)
+  {
+    if (!options->show_icon)
+    {
+      options->show_icon = TRUE;
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (configure->display_icon), TRUE);
+      update_sensitivity (configure);
+    }
+  }
+}
+
+
+
+static void
+check_button_changed (GtkWidget *button, const CpuFreqPluginConfigure *configure)
+{
+  CpuFreqPluginOptions *options = cpuFreq->options;
+
+  if (button == configure->display_icon)
+    options->show_icon = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+
+  else if (button == configure->display_freq)
+    options->show_label_freq = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+
+  else if (button == configure->display_governor)
+    options->show_label_governor = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+
+  else if (button == configure->icon_color_freq)
+    options->icon_color_freq = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+
+  else if (button == configure->keep_compact)
+    options->keep_compact = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+
+  else if (button == configure->one_line)
+    options->one_line = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+
+  update_sensitivity (configure);
+  validate_configuration (configure);
+
+  cpufreq_prepare_label ();
+  cpufreq_update_icon ();
   cpufreq_update_plugin (TRUE);
 }
 
@@ -98,7 +127,6 @@ static gboolean
 button_fontname_clicked(GtkWidget *button, CpuFreqPluginConfigure *configure)
 {
   GtkWidget *fc;
-  gchar *fontname;
   gint result;
 
   fc = gtk_font_chooser_dialog_new (_("Select font"),
@@ -111,13 +139,13 @@ button_fontname_clicked(GtkWidget *button, CpuFreqPluginConfigure *configure)
 
   if (result == GTK_RESPONSE_OK || result == GTK_RESPONSE_ACCEPT)
   {
-    fontname = gtk_font_chooser_get_font (GTK_FONT_CHOOSER (fc));
+    gchar *fontname = gtk_font_chooser_get_font (GTK_FONT_CHOOSER (fc));
 
     if (fontname != NULL)
     {
       gtk_button_set_label(GTK_BUTTON(button), fontname);
-      g_free (cpuFreq->options->fontname);
-      cpuFreq->options->fontname = fontname;
+      cpufreq_set_font (fontname);
+      g_free (fontname);
     }
 
     button_fontname_update(button, TRUE);
@@ -139,8 +167,7 @@ button_fontname_pressed(GtkWidget *button, GdkEventButton *event,
   /* right mouse click clears the font name and resets the button */
   if (event->button == 3 && cpuFreq->options->fontname)
   {
-    g_free (cpuFreq->options->fontname);
-    cpuFreq->options->fontname = NULL;
+    cpufreq_set_font (NULL);
     button_fontname_update(button, TRUE);
     return TRUE;
   }
@@ -151,16 +178,58 @@ button_fontname_pressed(GtkWidget *button, GdkEventButton *event,
 
 
 
+static void
+button_fontcolor_update(GtkWidget *button, gboolean update_plugin)
+{
+  if (cpuFreq->options->fontcolor == NULL)
+  {
+    GdkRGBA color = {};
+    gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (button), &color);
+    gtk_widget_set_tooltip_text (button, NULL);
+  }
+  else
+  {
+    gtk_widget_set_tooltip_text (button, _("Right-click to revert to the default color"));
+  }
+
+  if (update_plugin)
+    cpufreq_update_plugin (TRUE);
+}
+
+
+
 static void
 button_fontcolor_clicked (GtkWidget *button, void *data)
 {
-  GdkRGBA *color = g_new0 (GdkRGBA, 1);
-
-  gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (button), color);
+  GdkRGBA color;
+  gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (button), &color);
   g_free (cpuFreq->options->fontcolor);
-  cpuFreq->options->fontcolor = gdk_rgba_to_string (color);
-  g_free (color);
-  cpufreq_update_plugin (TRUE);
+  cpuFreq->options->fontcolor = NULL;
+  if (color.alpha != 0)
+    cpuFreq->options->fontcolor = gdk_rgba_to_string (&color);
+  button_fontcolor_update (button, TRUE);
+}
+
+
+
+static gboolean
+button_fontcolor_pressed(GtkWidget *button, GdkEventButton *event,
+                         CpuFreqPluginConfigure *configure)
+{
+  if (event->type != GDK_BUTTON_PRESS)
+    return FALSE;
+
+  /* right mouse click clears the font color and resets the button */
+  if (event->button == 3 && cpuFreq->options->fontcolor)
+  {
+    g_free (cpuFreq->options->fontcolor);
+    cpuFreq->options->fontcolor = NULL;
+    button_fontcolor_update(button, TRUE);
+    return TRUE;
+  }
+
+  /* left mouse click will be handled in a different function */
+  return FALSE;
 }
 
 
@@ -235,7 +304,7 @@ cpufreq_configure (XfcePanelPlugin *plugin)
   GtkWidget *spinner, *button;
   GtkSizeGroup *sg0;
   CpuFreqPluginConfigure *configure;
-  GdkRGBA *color;
+  GdkRGBA color = {};
 
   configure = g_new0 (CpuFreqPluginConfigure, 1);
 
@@ -244,7 +313,7 @@ cpufreq_configure (XfcePanelPlugin *plugin)
   dialog = xfce_titled_dialog_new_with_mixed_buttons (_("Configure CPU Frequency Monitor"),
     GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (plugin))),
     GTK_DIALOG_DESTROY_WITH_PARENT,
-    "window-close", _("_Close"), GTK_RESPONSE_OK,
+    "window-close-symbolic", _("_Close"), GTK_RESPONSE_OK,
     NULL);
 
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
@@ -318,7 +387,7 @@ cpufreq_configure (XfcePanelPlugin *plugin)
   gtk_container_add (GTK_CONTAINER (align), vbox);
 
   /* font settings */
-  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  hbox = configure->fontname_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
   label = gtk_label_new_with_mnemonic (_("_Font:"));
@@ -337,7 +406,7 @@ cpufreq_configure (XfcePanelPlugin *plugin)
   button_fontname_update (button, FALSE);
 
   /* font color */
-  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  hbox = configure->fontcolor_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
   label = gtk_label_new_with_mnemonic (_("_Font color:"));
@@ -346,15 +415,17 @@ cpufreq_configure (XfcePanelPlugin *plugin)
   gtk_label_set_xalign (GTK_LABEL (label), 0);
   gtk_size_group_add_widget (sg0, label);
 
-  color = g_new0 (GdkRGBA, 1);
-  gdk_rgba_parse (color, options->fontcolor ? options->fontcolor : "#000000");
+  if (options->fontcolor)
+    gdk_rgba_parse (&color, options->fontcolor);
 
-  button = configure->fontcolor = gtk_color_button_new_with_rgba (color);
+  button = configure->fontcolor = gtk_color_button_new_with_rgba (&color);
   gtk_color_button_set_title (GTK_COLOR_BUTTON (button), _("Select font color"));
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
   g_signal_connect (button, "color-set", G_CALLBACK (button_fontcolor_clicked), NULL);
-  g_free (color);
+  g_signal_connect (button, "button_press_event",
+                    G_CALLBACK (button_fontcolor_pressed), configure);
+  button_fontcolor_update (button, FALSE);
 
   /* which cpu to show in panel */
   {
@@ -465,8 +536,11 @@ cpufreq_configure (XfcePanelPlugin *plugin)
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), options->show_icon);
   g_signal_connect (G_OBJECT (button), "toggled", G_CALLBACK (check_button_changed), configure);
-  if (!options->show_label_freq && !options->show_label_governor)
-    gtk_widget_set_sensitive (configure->display_icon, FALSE);
+
+  button = configure->icon_color_freq = gtk_check_button_new_with_mnemonic (_("Adjust CPU icon color according to frequency"));
+  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), options->icon_color_freq);
+  g_signal_connect (G_OBJECT (button), "toggled", G_CALLBACK (check_button_changed), configure);
 
   button = configure->display_freq = gtk_check_button_new_with_mnemonic (_("Show CPU fre_quency"));
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -480,6 +554,9 @@ cpufreq_configure (XfcePanelPlugin *plugin)
 
   g_signal_connect(G_OBJECT (dialog), "response", G_CALLBACK(cpufreq_configure_response), configure);
 
+  update_sensitivity (configure);
+  validate_configuration (configure);
+
   g_object_unref (sg0);
   gtk_widget_show_all (dialog);
 }
