@@ -24,18 +24,19 @@
 
 typedef struct
 {
+  GtkWidget *display_cpu;
   GtkWidget *display_icon;
   GtkWidget *display_freq;
   GtkWidget *display_governor;
-  GtkWidget *display_cpu;
+  GtkWidget *icon_color_freq;
   GtkWidget *monitor_timeout;
   GtkWidget *combo_cpu;
   GtkWidget *combo_unit;
   GtkWidget *spinner_timeout;
   GtkWidget *keep_compact;
   GtkWidget *one_line;
-  GtkWidget *fontname;
-  GtkWidget *fontcolor;
+  GtkWidget *fontcolor, *fontcolor_hbox;
+  GtkWidget *fontname, *fontname_hbox;
 } CpuFreqPluginConfigure;
 
 G_BEGIN_DECLS
