@@ -39,7 +39,7 @@
 static void
 cpufreq_overview_add (CpuInfo *cpu, guint cpu_number, GtkWidget *dialog_hbox)
 {
-  gint i = 0, j;
+  gint i = 0;
   gchar *text;
   GtkWidget *hbox, *dialog_vbox, *combo, *label, *icon;
   GtkSizeGroup *sg0, *sg1;
@@ -108,6 +108,7 @@ cpufreq_overview_add (CpuInfo *cpu, guint cpu_number, GtkWidget *dialog_hbox)
 
   if (cpu->available_freqs != NULL) /* Linux 2.6 with scaling support */
   {
+    gint j;
     combo = gtk_combo_box_text_new ();
     gtk_size_group_add_widget (sg1, combo);
     gtk_box_pack_end (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
@@ -127,7 +128,7 @@ cpufreq_overview_add (CpuInfo *cpu, guint cpu_number, GtkWidget *dialog_hbox)
     }
     gtk_combo_box_set_active (GTK_COMBO_BOX (combo), i);
   }
-  else if (cpu->cur_freq && cpu->min_freq && cpu->max_freq) /* Linux 2.4 with scaling support */
+  else if (cpu->cur_freq && cpu->min_freq && cpu->max_freq_nominal) /* Linux 2.4 with scaling support */
   {
     combo = gtk_combo_box_text_new ();
     gtk_size_group_add_widget (sg1, combo);
@@ -137,7 +138,7 @@ cpufreq_overview_add (CpuInfo *cpu, guint cpu_number, GtkWidget *dialog_hbox)
     gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), text);
     g_free (text);
 
-    text = cpufreq_get_human_readable_freq (cpu->max_freq, unit);
+    text = cpufreq_get_human_readable_freq (cpu->max_freq_nominal, unit);
     gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), text);
     g_free (text);
 
@@ -164,6 +165,8 @@ cpufreq_overview_add (CpuInfo *cpu, guint cpu_number, GtkWidget *dialog_hbox)
   /* display list of available governors */
   if (cpu->available_governors != NULL) /* Linux 2.6 and cpu scaling support */
   {
+    gint j;
+
     hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, BORDER);
     gtk_box_pack_start (GTK_BOX (dialog_vbox), hbox, FALSE, FALSE, 0);
 
@@ -176,9 +179,9 @@ cpufreq_overview_add (CpuInfo *cpu, guint cpu_number, GtkWidget *dialog_hbox)
     combo = gtk_combo_box_text_new ();
     gtk_size_group_add_widget (sg1, combo);
     gtk_box_pack_end (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
+
     list = g_list_first (cpu->available_governors);
     j = 0;
-
     while (list)
     {
       gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), list->data);
@@ -255,7 +258,7 @@ cpufreq_overview (GtkWidget *widget, GdkEventButton *ev, CpuFreqPlugin *cpufreq)
   dialog = xfce_titled_dialog_new_with_mixed_buttons (_("CPU Information"),
     GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (cpufreq->plugin))),
     GTK_DIALOG_DESTROY_WITH_PARENT,
-    "window-close", _("_Close"), GTK_RESPONSE_OK,
+    "window-close-symbolic", _("_Close"), GTK_RESPONSE_OK,
     NULL);
 
   xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (dialog),
