@@ -27,6 +27,7 @@
 #endif
 
 #include <libxfce4ui/libxfce4ui.h>
+#include <math.h>
 #include <string.h>
 
 #include "xfce4-cpufreq-plugin.h"
@@ -38,70 +39,7 @@
 #include "xfce4-cpufreq-linux.h"
 #endif /* __linux__ */
 
-CpuFreqPlugin *cpuFreq;
-
-static void
-cpufreq_label_set_font (void)
-{
-  gchar *css = NULL, *css_font = NULL, *css_color = NULL;
-  PangoFontDescription *font;
-  GtkWidget *label;
-
-  if (G_UNLIKELY (cpuFreq->label_orNull == NULL))
-    return;
-
-  label = cpuFreq->label_orNull;
-
-  if (cpuFreq->options->fontname)
-  {
-    font = pango_font_description_from_string(cpuFreq->options->fontname);
-
-    css_font = g_strdup_printf("font-family: %s; font-size: %dpt; font-style: %s; font-weight: %s;",
-      pango_font_description_get_family (font),
-      pango_font_description_get_size (font) / PANGO_SCALE,
-      (pango_font_description_get_style (font) == PANGO_STYLE_ITALIC ||
-      pango_font_description_get_style (font) == PANGO_STYLE_OBLIQUE) ? "italic" : "normal",
-      (pango_font_description_get_weight (font) >= PANGO_WEIGHT_BOLD) ? "bold" : "normal");
-
-    pango_font_description_free (font);
-  }
-
-  if (cpuFreq->options->fontcolor)
-    css_color = g_strdup_printf ("color: %s;", cpuFreq->options->fontcolor);
-
-  if (css_font && css_color)
-    css = g_strdup_printf ("label { %s %s }", css_font, css_color);
-  else if (css_font)
-    css = g_strdup_printf ("label { %s }", css_font);
-  else if (css_color)
-    css = g_strdup_printf ("label { %s }", css_color);
-
-  if (css)
-  {
-    GtkCssProvider *provider;
-
-    if (cpuFreq->label_css_provider)
-    {
-      gtk_style_context_remove_provider (
-        GTK_STYLE_CONTEXT (gtk_widget_get_style_context (label)),
-        GTK_STYLE_PROVIDER (cpuFreq->label_css_provider));
-      cpuFreq->label_css_provider = NULL;
-    }
-
-    provider = gtk_css_provider_new ();
-
-    gtk_css_provider_load_from_data (provider, css, -1, NULL);
-    gtk_style_context_add_provider (
-      GTK_STYLE_CONTEXT (gtk_widget_get_style_context (label)),
-      GTK_STYLE_PROVIDER (provider), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-
-    cpuFreq->label_css_provider = provider;
-  }
-
-  g_free (css);
-  g_free (css_font);
-  g_free (css_color);
-}
+CpuFreqPlugin *cpuFreq = NULL;
 
 
 
@@ -182,28 +120,38 @@ cpufreq_cpus_calc_min (void)
 {
   gchar *const governors = cpufreq_governors ();
   gchar *const old_governor = cpuFreq->cpu_min ? g_strdup (cpuFreq->cpu_min->cur_governor) : g_strdup ("");
-  guint freq = 0;
+  guint freq = G_MAXUINT, max_freq_measured = G_MAXUINT, max_freq_nominal = G_MAXUINT, min_freq = G_MAXUINT;
+  guint count = 0, i;
 
-  for (guint i = 0; i < cpuFreq->cpus->len; i++)
+  for (i = 0; i < cpuFreq->cpus->len; i++)
   {
     CpuInfo *cpu = g_ptr_array_index (cpuFreq->cpus, i);
 
     if (!cpu->online)
       continue;
 
-    if (freq > cpu->cur_freq || i == 0)
-      freq = cpu->cur_freq;
+    freq = MIN (freq, cpu->cur_freq);
+    max_freq_measured = MIN (max_freq_measured, cpu->max_freq_measured);
+    max_freq_nominal = MIN (max_freq_nominal, cpu->max_freq_nominal);
+    min_freq = MIN (min_freq, cpu->min_freq);
+    count++;
   }
 
+  if (count == 0)
+    freq = max_freq_measured = max_freq_nominal = min_freq = 0;
+
   cpuinfo_free (cpuFreq->cpu_min);
   cpuFreq->cpu_min = g_new0 (CpuInfo, 1);
   cpuFreq->cpu_min->cur_freq = freq;
   cpuFreq->cpu_min->cur_governor = governors ? governors : g_strdup (_("current min"));
+  cpuFreq->cpu_min->max_freq_measured = max_freq_measured;
+  cpuFreq->cpu_min->max_freq_nominal = max_freq_nominal;
+  cpuFreq->cpu_min->min_freq = min_freq;
 
   if (cpuFreq->options->show_label_governor && strcmp(cpuFreq->cpu_min->cur_governor, old_governor) != 0)
   {
+    cpuFreq->label.reset_size = TRUE;
     cpuFreq->layout_changed = TRUE;
-    cpuFreq->label_max_width = -1;
   }
 
   g_free (old_governor);
@@ -217,9 +165,10 @@ cpufreq_cpus_calc_avg (void)
 {
   gchar *const governors = cpufreq_governors ();
   gchar *const old_governor = cpuFreq->cpu_avg ? g_strdup (cpuFreq->cpu_avg->cur_governor) : g_strdup ("");
-  guint freq = 0, count = 0;
+  guint freq = 0, max_freq_measured = 0, max_freq_nominal = 0, min_freq = 0;
+  guint count = 0, i;
 
-  for (guint i = 0; i < cpuFreq->cpus->len; i++)
+  for (i = 0; i < cpuFreq->cpus->len; i++)
   {
     CpuInfo *cpu = g_ptr_array_index (cpuFreq->cpus, i);
 
@@ -227,21 +176,32 @@ cpufreq_cpus_calc_avg (void)
       continue;
 
     freq += cpu->cur_freq;
+    max_freq_measured += cpu->max_freq_measured;
+    max_freq_nominal += cpu->max_freq_nominal;
+    min_freq += cpu->min_freq;
     count++;
   }
 
-  if (count > 0)
+  if (count != 0)
+  {
     freq /= count;
+    max_freq_measured /= count;
+    max_freq_nominal /= count;
+    min_freq /= count;
+  }
 
   cpuinfo_free (cpuFreq->cpu_avg);
   cpuFreq->cpu_avg = g_new0 (CpuInfo, 1);
   cpuFreq->cpu_avg->cur_freq = freq;
   cpuFreq->cpu_avg->cur_governor = governors ? governors : g_strdup (_("current avg"));
+  cpuFreq->cpu_avg->max_freq_measured = max_freq_measured;
+  cpuFreq->cpu_avg->max_freq_nominal = max_freq_nominal;
+  cpuFreq->cpu_avg->min_freq = min_freq;
 
   if (cpuFreq->options->show_label_governor && strcmp(cpuFreq->cpu_avg->cur_governor, old_governor) != 0)
   {
+    cpuFreq->label.reset_size = TRUE;
     cpuFreq->layout_changed = TRUE;
-    cpuFreq->label_max_width = -1;
   }
 
   g_free (old_governor);
@@ -255,28 +215,34 @@ cpufreq_cpus_calc_max (void)
 {
   gchar *const governors = cpufreq_governors ();
   gchar *const old_governor = cpuFreq->cpu_max ? g_strdup (cpuFreq->cpu_max->cur_governor) : g_strdup ("");
-  guint freq = 0;
+  guint freq = 0, max_freq_measured = 0, max_freq_nominal = 0, min_freq = 0;
+  guint i;
 
-  for (guint i = 0; i < cpuFreq->cpus->len; i++)
+  for (i = 0; i < cpuFreq->cpus->len; i++)
   {
     CpuInfo *cpu = g_ptr_array_index (cpuFreq->cpus, i);
 
     if (!cpu->online)
       continue;
 
-    if (freq < cpu->cur_freq)
-      freq = cpu->cur_freq;
+    freq = MAX (freq, cpu->cur_freq);
+    max_freq_measured = MAX (max_freq_measured, cpu->max_freq_measured);
+    max_freq_nominal = MAX (max_freq_nominal, cpu->max_freq_nominal);
+    min_freq = MAX (min_freq, cpu->min_freq);
   }
 
   cpuinfo_free (cpuFreq->cpu_max);
   cpuFreq->cpu_max = g_new0 (CpuInfo, 1);
   cpuFreq->cpu_max->cur_freq = freq;
   cpuFreq->cpu_max->cur_governor = governors ? governors : g_strdup (_("current max"));
+  cpuFreq->cpu_max->max_freq_measured = max_freq_measured;
+  cpuFreq->cpu_max->max_freq_nominal = max_freq_nominal;
+  cpuFreq->cpu_max->min_freq = min_freq;
 
   if (cpuFreq->options->show_label_governor && strcmp(cpuFreq->cpu_max->cur_governor, old_governor) != 0)
   {
+    cpuFreq->label.reset_size = TRUE;
     cpuFreq->layout_changed = TRUE;
-    cpuFreq->label_max_width = -1;
   }
 
   g_free (old_governor);
@@ -285,84 +251,61 @@ cpufreq_cpus_calc_max (void)
 
 
 
-static gboolean
-cpufreq_update_label (CpuInfo *cpu)
+static void
+cpufreq_update_label (const CpuInfo *cpu)
 {
+  const CpuFreqPluginOptions *const options = cpuFreq->options;
   GtkWidget *label_widget;
-  GtkRequisition label_size;
   gchar *label, *freq;
-  gint both;
+  gboolean both;
 
-  if (!cpuFreq->label_orNull)
-    return TRUE;
+  if (!cpuFreq->label.draw_area)
+    return;
 
-  label_widget = cpuFreq->label_orNull;
+  label_widget = cpuFreq->label.draw_area;
 
-  if (!cpuFreq->options->show_label_governor && !cpuFreq->options->show_label_freq) {
+  if (!options->show_label_governor && !options->show_label_freq)
+  {
     gtk_widget_hide (label_widget);
-    return TRUE;
+    return;
   }
 
   both = cpu->cur_governor != NULL &&
-    cpuFreq->options->show_label_freq &&
-    cpuFreq->options->show_label_governor;
+    options->show_label_freq &&
+    options->show_label_governor;
 
-  freq = cpufreq_get_human_readable_freq (cpu->cur_freq, cpuFreq->options->unit);
+  freq = cpufreq_get_human_readable_freq (cpu->cur_freq, options->unit);
   label = g_strconcat
-    (cpuFreq->options->show_label_freq ? freq : "",
-     both ? (cpuFreq->options->one_line ? " " : "\n") : "",
+    (options->show_label_freq ? freq : "",
+     both ? (options->one_line ? " " : "\n") : "",
      cpu->cur_governor != NULL &&
-     cpuFreq->options->show_label_governor ? cpu->cur_governor : "",
+     options->show_label_governor ? cpu->cur_governor : "",
      NULL);
 
-  gtk_label_set_text (GTK_LABEL (label_widget), label);
-
-  if (strcmp(label, ""))
+  if (*label != '\0')
   {
-    if (cpuFreq->panel_mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL)
-      gtk_label_set_angle (GTK_LABEL(label_widget), -90);
-    else
-      gtk_label_set_angle (GTK_LABEL(label_widget), 0);
+    if (!gtk_widget_is_visible (label_widget))
+      gtk_widget_show (label_widget);
 
-    gtk_widget_show (label_widget);
-
-    /* Set label width to max width if smaller to avoid panel
-       resizing/jumping (see bug #10385). */
-    gtk_widget_get_preferred_size (label_widget, NULL, &label_size);
-    if (cpuFreq->panel_mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL)
+    if (!cpuFreq->label.text || strcmp (cpuFreq->label.text, label) != 0)
     {
-      if (label_size.height < cpuFreq->label_max_width)
-      {
-        gtk_widget_set_size_request (label_widget, -1, cpuFreq->label_max_width);
-      }
-      else if (label_size.height > cpuFreq->label_max_width)
-      {
-        cpuFreq->label_max_width = label_size.height;
-        cpuFreq->layout_changed = TRUE;
-      }
-    }
-    else
-    {
-      if (label_size.width < cpuFreq->label_max_width)
-      {
-        gtk_widget_set_size_request (label_widget, cpuFreq->label_max_width, -1);
-      }
-      else if (label_size.width > cpuFreq->label_max_width)
-      {
-        cpuFreq->label_max_width = label_size.width;
-        cpuFreq->layout_changed = TRUE;
-      }
+      g_free (cpuFreq->label.text);
+      cpuFreq->label.text = g_strdup (label);
+      gtk_widget_queue_draw (label_widget);
     }
   }
   else
   {
+    if (cpuFreq->label.text)
+    {
+      g_free (cpuFreq->label.text);
+      cpuFreq->label.text = NULL;
+    }
     gtk_widget_hide (label_widget);
   }
 
   g_free (freq);
   g_free (label);
-
-  return TRUE;
 }
 
 
@@ -398,9 +341,9 @@ cpufreq_widgets_layout (void)
   }
 
   /* check if the label fits below the icon, else put them side by side */
-  if (cpuFreq->label_orNull && ! hide_label)
+  if (cpuFreq->label.draw_area && !hide_label)
   {
-    gtk_widget_get_preferred_size (cpuFreq->label_orNull, NULL, &label_size);
+    gtk_widget_get_preferred_size (cpuFreq->label.draw_area, NULL, &label_size);
     lw = label_size.width;
     lh = label_size.height;
   }
@@ -434,21 +377,13 @@ cpufreq_widgets_layout (void)
   {
       if (cpuFreq->icon)
         gtk_widget_set_halign (cpuFreq->icon, GTK_ALIGN_CENTER);
-      if (cpuFreq->label_orNull)
-        gtk_widget_set_halign (cpuFreq->label_orNull, GTK_ALIGN_CENTER);
     }
     else
     {
       if (cpuFreq->icon)
         gtk_widget_set_valign (cpuFreq->icon, GTK_ALIGN_CENTER);
-      if (cpuFreq->label_orNull)
-        gtk_widget_set_valign (cpuFreq->label_orNull, GTK_ALIGN_CENTER);
     }
 
-    if (cpuFreq->label_orNull)
-      gtk_label_set_justify (GTK_LABEL (cpuFreq->label_orNull),
-        resized ? GTK_JUSTIFY_CENTER : GTK_JUSTIFY_LEFT);
-
     if (cpuFreq->icon)
       gtk_box_set_child_packing (GTK_BOX (cpuFreq->box),
         cpuFreq->icon, FALSE, FALSE, 0, GTK_PACK_START);
@@ -462,34 +397,25 @@ cpufreq_widgets_layout (void)
         gtk_widget_set_halign (cpuFreq->icon, GTK_ALIGN_CENTER);
         gtk_widget_set_valign (cpuFreq->icon, GTK_ALIGN_END);
       }
-
-      if (cpuFreq->label_orNull)
-        gtk_widget_set_halign (cpuFreq->label_orNull, GTK_ALIGN_CENTER);
     }
     else
     {
       if (cpuFreq->icon)
         gtk_widget_set_valign (cpuFreq->icon, GTK_ALIGN_CENTER);
 
-      if (cpuFreq->label_orNull)
-      {
-        gtk_widget_set_halign (cpuFreq->label_orNull, GTK_ALIGN_END);
-        gtk_widget_set_valign (cpuFreq->label_orNull, GTK_ALIGN_CENTER);
-      }
       pos = resized ? 1 : 0;
     }
 
-    if (cpuFreq->label_orNull)
-      gtk_label_set_justify (GTK_LABEL (cpuFreq->label_orNull),
-        resized ? GTK_JUSTIFY_LEFT : GTK_JUSTIFY_CENTER);
-
     if (cpuFreq->icon)
       gtk_box_set_child_packing (GTK_BOX (cpuFreq->box),
         cpuFreq->icon, TRUE, TRUE, 0, GTK_PACK_START);
   }
 
-  if (cpuFreq->label_orNull)
-    gtk_box_reorder_child (GTK_BOX (cpuFreq->box), cpuFreq->label_orNull, pos);
+  if (cpuFreq->label.draw_area)
+  {
+    gtk_box_reorder_child (GTK_BOX (cpuFreq->box), cpuFreq->label.draw_area, pos);
+    gtk_widget_queue_draw (cpuFreq->label.draw_area);
+  }
 
   cpuFreq->layout_changed = FALSE;
 }
@@ -535,33 +461,144 @@ cpufreq_current_cpu (void)
 
 
 
+static void
+cpufreq_update_pixmap (CpuInfo *cpu)
+{
+  const gdouble min_range = 100*1000; /* frequency in kHz */
+  gdouble freq_99, normalized_freq, range;
+  gint i, index, total_count;
+  GdkPixbuf *pixmap;
+
+  if (G_UNLIKELY (!cpuFreq->icon || !cpuFreq->base_icon))
+    return;
+
+  /* Note:
+   *   max_freq_nominal can have values that are outside
+   *   of the actual maximum frequency of the CPU.
+   *   For example, Linux kernel 5.10.17 reports 5554687 kHz
+   *   for some CPU cores on Ryzen 3700X, but the actual top
+   *   frequency of this CPU is about 4.4 GHz.
+   *   Therefore, the following algorithm relies on measured
+   *   frequencies only.
+   */
+
+  /* Compute the 99th percentile of all measured frequencies
+   * and use it as the "true" maximum frequency. This is required
+   * because in a small percentage of cases the current CPU
+   * frequency reported by Linux can be unrealistic, for example
+   * 4.59 GHz on a CPU that has an actual maximum frequency
+   * of about 4.4 GHz.
+   */
+  total_count = 0;
+  for (i = 0; i < (gint) G_N_ELEMENTS (cpuFreq->freq_hist); i++)
+    total_count += cpuFreq->freq_hist[i];
+  if (total_count * 0.01 < 1)
+  {
+    /* Not enough data to reliably compute the percentile,
+     * resort to a value that isn't based on statistics */
+    freq_99 = MAX (cpu->max_freq_nominal, cpu->max_freq_measured);
+  }
+  else
+  {
+    gint percentile_2 = total_count * 0.01;
+    for (i = G_N_ELEMENTS (cpuFreq->freq_hist) - 1; i >= 0; i--)
+    {
+      guint16 count = cpuFreq->freq_hist[i];
+      if (count < percentile_2)
+        percentile_2 -= count;
+      else
+      {
+        freq_99 = FREQ_HIST_MIN + i * ((gdouble) (FREQ_HIST_MAX - FREQ_HIST_MIN) / FREQ_HIST_BINS);
+        break;
+      }
+    }
+    if (G_UNLIKELY (i == -1))
+      freq_99 = cpu->max_freq_measured;
+  }
+
+  range = freq_99 - cpu->min_freq;
+  if (cpu->cur_freq > cpu->min_freq && range >= min_range)
+    normalized_freq = (cpu->cur_freq - cpu->min_freq) / range;
+  else
+    normalized_freq = 0;
+  index = round (normalized_freq * (G_N_ELEMENTS (cpuFreq->icon_pixmaps) - 1));
+  if (G_UNLIKELY (index < 0))
+    index = 0;
+  if (index >= (gint) G_N_ELEMENTS (cpuFreq->icon_pixmaps))
+    /* This codepath is expected to be reached in 100-99=1% of cases */
+    index = G_N_ELEMENTS (cpuFreq->icon_pixmaps) - 1;
+
+  pixmap = cpuFreq->icon_pixmaps[index];
+  if (!pixmap)
+  {
+    guchar color, *pixels;
+    gsize p, plength;
+    int n_channels;
+
+    color = (guchar) (255 * index / (G_N_ELEMENTS (cpuFreq->icon_pixmaps) - 1));
+
+    pixmap = gdk_pixbuf_copy (cpuFreq->base_icon);
+    if (G_UNLIKELY (!pixmap))
+      return;
+
+    pixels = gdk_pixbuf_get_pixels (pixmap);
+    plength = gdk_pixbuf_get_byte_length (pixmap);
+    n_channels = gdk_pixbuf_get_n_channels (pixmap);
+    if (G_UNLIKELY (n_channels <= 0))
+    {
+      g_object_unref (G_OBJECT (pixmap));
+      return;
+    }
+
+    for (p = 0; p+2 < plength; p += n_channels)
+    {
+      gint delta1 = abs ((gint) pixels[p] - (gint) pixels[p+1]);
+      gint delta2 = abs ((gint) pixels[p] - (gint) pixels[p+2]);
+      if (delta1 < 10 && delta2 < 10)
+        pixels[p] = MAX (pixels[p], color);
+    }
+
+    cpuFreq->icon_pixmaps[index] = pixmap;
+  }
+
+  if (cpuFreq->current_icon_pixmap != pixmap)
+  {
+    cpuFreq->current_icon_pixmap = pixmap;
+    gdk_pixbuf_copy_area (pixmap, 0, 0,
+                          gdk_pixbuf_get_width (pixmap),
+                          gdk_pixbuf_get_height (pixmap),
+                          gtk_image_get_pixbuf (GTK_IMAGE (cpuFreq->icon)),
+                          0, 0);
+    g_signal_emit_by_name (cpuFreq->icon, "style-updated");
+  }
+}
+
+
+
 gboolean
 cpufreq_update_plugin (gboolean reset_label_size)
 {
   CpuInfo *cpu;
-  gboolean ret;
 
   cpu = cpufreq_current_cpu ();
   if (!cpu)
-      return FALSE;
+    return FALSE;
 
   if (reset_label_size)
   {
-    cpuFreq->label_max_width = -1;
-    if (cpuFreq->label_orNull)
-      gtk_widget_set_size_request (cpuFreq->label_orNull, -1, -1);
+    cpuFreq->label.reset_size = TRUE;
     cpuFreq->layout_changed = TRUE;
   }
 
-  ret = cpufreq_update_label (cpu);
+  cpufreq_update_label (cpu);
+
+  if (cpuFreq->options->icon_color_freq)
+    cpufreq_update_pixmap (cpu);
 
   if (cpuFreq->layout_changed)
-  {
-    cpufreq_label_set_font ();
     cpufreq_widgets_layout ();
-  }
 
-  return ret;
+  return TRUE;
 }
 
 static gboolean
@@ -574,17 +611,17 @@ cpufreq_update_tooltip (GtkWidget *widget,
 {
   const CpuFreqPluginOptions *const options = cpuFreq->options;
   CpuInfo *cpu;
-  gchar *tooltip_msg, *freq = NULL;
+  gchar *tooltip_msg;
 
   cpu = cpufreq_current_cpu ();
 
-  if (G_UNLIKELY(cpu == NULL))
+  if (G_UNLIKELY (cpu == NULL))
   {
     tooltip_msg = g_strdup (_("No CPU information available."));
   }
   else
   {
-    freq = cpufreq_get_human_readable_freq (cpu->cur_freq, options->unit);
+    gchar *freq = cpufreq_get_human_readable_freq (cpu->cur_freq, options->unit);
     if (options->show_label_governor && options->show_label_freq)
       tooltip_msg = g_strdup_printf (ngettext ("%d cpu available",
         "%d cpus available", cpuFreq->cpus->len), cpuFreq->cpus->len);
@@ -603,13 +640,11 @@ cpufreq_update_tooltip (GtkWidget *widget,
          cpu->cur_governor != NULL &&
          !options->show_label_governor ? cpu->cur_governor : "",
          NULL);
+    g_free (freq);
   }
 
   gtk_tooltip_set_text (tooltip, tooltip_msg);
-
-  g_free (freq);
   g_free (tooltip_msg);
-
   return TRUE;
 }
 
@@ -627,6 +662,30 @@ cpufreq_restart_timeout (void)
 
 
 
+void
+cpufreq_set_font (const gchar *fontname_or_null)
+{
+  if (cpuFreq->label.font_desc)
+  {
+    pango_font_description_free (cpuFreq->label.font_desc);
+    cpuFreq->label.font_desc = NULL;
+  }
+
+  if (fontname_or_null)
+  {
+    g_free (cpuFreq->options->fontname);
+    cpuFreq->options->fontname = g_strdup (fontname_or_null);
+    cpuFreq->label.font_desc = pango_font_description_from_string (fontname_or_null);
+  }
+  else
+  {
+    g_free (cpuFreq->options->fontname);
+    cpuFreq->options->fontname = NULL;
+  }
+}
+
+
+
 static void
 cpufreq_mode_changed (XfcePanelPlugin *plugin,
                       XfcePanelPluginMode mode,
@@ -639,25 +698,53 @@ cpufreq_mode_changed (XfcePanelPlugin *plugin,
 
 
 
-void
-cpufreq_update_icon (CpuFreqPlugin *cpufreq)
+static void
+cpufreq_destroy_icons (void)
 {
-  if (cpufreq->icon)
+  gsize i;
+
+  if (cpuFreq->icon)
   {
-    gtk_widget_destroy (cpufreq->icon);
-    cpufreq->icon = NULL;
+    gtk_widget_destroy (cpuFreq->icon);
+    cpuFreq->icon = NULL;
   }
 
-  if (cpufreq->options->show_icon)
+  if (cpuFreq->base_icon)
   {
-    GdkPixbuf *buf, *scaled;
+    g_object_unref (G_OBJECT (cpuFreq->base_icon));
+    cpuFreq->base_icon = NULL;
+  }
+
+  for (i = 0; i < G_N_ELEMENTS (cpuFreq->icon_pixmaps); i++)
+    if (cpuFreq->icon_pixmaps[i])
+    {
+      g_object_unref (G_OBJECT (cpuFreq->icon_pixmaps[i]));
+      cpuFreq->icon_pixmaps[i] = NULL;
+    }
+
+  cpuFreq->current_icon_pixmap = NULL;
+}
+
+
+
+void
+cpufreq_update_icon (void)
+{
+  const CpuFreqPluginOptions *options = cpuFreq->options;
+
+  cpufreq_destroy_icons ();
+
+  /* Load and scale the icon */
+  if (options->show_icon)
+  {
+    GdkPixbuf *buf;
     gint icon_size;
 
     icon_size = cpuFreq->panel_size / cpuFreq->panel_rows;
 
-    if (cpufreq->options->keep_compact ||
-      (!cpufreq->options->show_label_freq &&
-       !cpufreq->options->show_label_governor))
+    if (options->keep_compact ||
+      (!options->show_label_freq &&
+       !options->show_label_governor))
       icon_size -= 4;
 
     buf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
@@ -665,44 +752,196 @@ cpufreq_update_icon (CpuFreqPlugin *cpufreq)
 
     if (buf)
     {
-      scaled = gdk_pixbuf_scale_simple (buf, icon_size, icon_size, GDK_INTERP_BILINEAR);
-      cpufreq->icon = gtk_image_new_from_pixbuf (scaled);
+      GdkPixbuf *scaled = gdk_pixbuf_scale_simple (buf, icon_size, icon_size, GDK_INTERP_BILINEAR);
+      if (G_LIKELY (scaled != NULL))
+      {
+        g_object_unref (G_OBJECT (buf));
+        buf = scaled;
+      }
+      cpuFreq->icon = gtk_image_new_from_pixbuf (buf);
+      cpuFreq->base_icon = gdk_pixbuf_copy (buf);
       g_object_unref (G_OBJECT (buf));
-      g_object_unref (G_OBJECT (scaled));
     }
     else
     {
-      cpufreq->icon = gtk_image_new_from_icon_name ("xfce4-cpufreq-plugin", GTK_ICON_SIZE_BUTTON);
+      cpuFreq->icon = gtk_image_new_from_icon_name ("xfce4-cpufreq-plugin", GTK_ICON_SIZE_BUTTON);
+      /* At this point: The storage type of the icon isn't GTK_IMAGE_PIXBUF */
     }
 
-    gtk_box_pack_start (GTK_BOX (cpufreq->box), cpufreq->icon, FALSE, FALSE, 0);
-    gtk_widget_show (cpufreq->icon);
+    if (G_LIKELY (cpuFreq->icon))
+    {
+      gtk_box_pack_start (GTK_BOX (cpuFreq->box), cpuFreq->icon, FALSE, FALSE, 0);
+      gtk_box_reorder_child (GTK_BOX (cpuFreq->box), cpuFreq->icon, 0);
+      gtk_widget_show (cpuFreq->icon);
+    }
   }
 }
 
 
 
-void
-cpufreq_prepare_label (CpuFreqPlugin *cpufreq)
+static void
+label_draw (GtkWidget *widget, cairo_t *cr, gpointer data)
 {
-  if (cpufreq->label_orNull)
+  GtkAllocation alloc;
+  GdkRGBA color;
+  PangoLayout *layout;
+  PangoContext *pango_context;
+  PangoRectangle extents;
+  GtkStyleContext *style_context;
+
+  if (G_UNLIKELY (!cpuFreq->label.text))
+    return;
+
+  cairo_save (cr);
+
+  gtk_widget_get_allocation (widget, &alloc);
+  pango_context = gtk_widget_get_pango_context (widget);
+  style_context = gtk_widget_get_style_context (widget);
+
+  if (cpuFreq->options->fontcolor)
   {
-    if (cpuFreq->label_css_provider)
+    gdk_rgba_parse (&color, cpuFreq->options->fontcolor);
+  }
+  else
+  {
+    gtk_style_context_get_color (style_context,
+                                 gtk_style_context_get_state (style_context),
+                                 &color);
+  }
+  gdk_cairo_set_source_rgba (cr, &color);
+
+  layout = pango_layout_new (pango_context);
+
+  if (cpuFreq->label.font_desc)
+    pango_layout_set_font_description (layout, cpuFreq->label.font_desc);
+
+  pango_layout_set_text (layout, cpuFreq->label.text, -1);
+
+  if (cpuFreq->panel_mode != XFCE_PANEL_PLUGIN_MODE_VERTICAL)
+  {
+    pango_layout_get_extents (layout, NULL, &extents);
+    if (alloc.width > PANGO_PIXELS_CEIL (extents.width))
     {
-      gtk_style_context_remove_provider (
-        GTK_STYLE_CONTEXT (gtk_widget_get_style_context (cpuFreq->label_orNull)),
-        GTK_STYLE_PROVIDER (cpuFreq->label_css_provider));
-      cpuFreq->label_css_provider = NULL;
+      double x0 = (double) extents.x / PANGO_SCALE;
+      double x1 = alloc.width / 2.0 - extents.width / 2.0 / PANGO_SCALE;
+      cairo_translate (cr, x1 - x0, 0);
+    }
+    if (alloc.height < PANGO_PIXELS_CEIL (extents.height))
+    {
+      double y0 = (double) extents.y / PANGO_SCALE;
+      double y1 = alloc.height / 2.0 - extents.height / 2.0 / PANGO_SCALE;
+      cairo_translate (cr, 0, y1 - y0);
     }
 
-    gtk_widget_destroy (cpufreq->label_orNull);
-    cpufreq->label_orNull = NULL;
+    /* Set label width to max width if smaller to avoid panel
+       resizing/jumping (see bug #10385). */
+    cpuFreq->label.reset_size |= alloc.width < PANGO_PIXELS_CEIL (extents.width);
+
+    if (cpuFreq->label.reset_size)
+    {
+      gtk_widget_set_size_request (widget,
+                                   PANGO_PIXELS_CEIL (extents.width),
+                                   PANGO_PIXELS_CEIL (extents.height));
+      cpuFreq->label.reset_size = FALSE;
+      cpuFreq->layout_changed = TRUE;
+    }
+  }
+  else
+  {
+    PangoRectangle non_transformed_extents;
+
+    /* rotate by 90° */
+    cairo_rotate (cr, M_PI_2);
+    cairo_translate (cr, 0, -alloc.width);
+    pango_cairo_update_layout (cr, layout);
+
+    pango_layout_get_extents (layout, NULL, &non_transformed_extents);
+    extents.x = non_transformed_extents.y;
+    extents.y = non_transformed_extents.x;
+    extents.width = non_transformed_extents.height;
+    extents.height = non_transformed_extents.width;
+    if (alloc.width < PANGO_PIXELS_CEIL (extents.width))
+    {
+      double x0 = (double) extents.x / PANGO_SCALE;
+      double x1 = alloc.width / 2.0 - extents.width / 2.0 / PANGO_SCALE;
+      /* cairo_translate: X and Y are swapped because of the rotation by 90° */
+      cairo_translate (cr, 0, x1 - x0);
+    }
+    if (alloc.height > PANGO_PIXELS_CEIL (extents.height))
+    {
+      double y0 = (double) extents.y / PANGO_SCALE;
+      double y1 = alloc.height / 2.0 - extents.height / 2.0 / PANGO_SCALE;
+      /* cairo_translate: X and Y are swapped because of the rotation by 90° */
+      cairo_translate (cr, y1 - y0, 0);
+    }
+
+    /* Set label height to max height if smaller to avoid panel
+       resizing/jumping (see bug #10385). */
+    cpuFreq->label.reset_size |= alloc.height < PANGO_PIXELS_CEIL (extents.height);
   }
 
+  if (cpuFreq->label.reset_size)
+  {
+    gtk_widget_set_size_request (widget,
+                                 PANGO_PIXELS_CEIL (extents.width),
+                                 PANGO_PIXELS_CEIL (extents.height));
+    cpuFreq->label.reset_size = FALSE;
+    cpuFreq->layout_changed = TRUE;
+  }
+
+  pango_cairo_show_layout (cr, layout);
+
+  g_object_unref (layout);
+  cairo_restore (cr);
+}
+
+static gboolean
+label_enter (GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  gtk_widget_set_state_flags (cpuFreq->button, GTK_STATE_FLAG_PRELIGHT, FALSE);
+  return FALSE;
+}
+
+static gboolean
+label_leave (GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+  gtk_widget_unset_state_flags (cpuFreq->button, GTK_STATE_FLAG_PRELIGHT);
+  return FALSE;
+}
+
+
+
+void
+cpufreq_prepare_label (void)
+{
   if (cpuFreq->options->show_label_freq || cpuFreq->options->show_label_governor)
   {
-    cpuFreq->label_orNull = gtk_label_new (NULL);
-    gtk_box_pack_start (GTK_BOX (cpufreq->box), cpuFreq->label_orNull, TRUE, TRUE, 0);
+    if (!cpuFreq->label.draw_area)
+    {
+      GtkWidget *draw_area = gtk_drawing_area_new ();
+      gtk_widget_add_events (draw_area, GDK_ALL_EVENTS_MASK);
+      g_signal_connect (draw_area, "draw", G_CALLBACK (label_draw), NULL);
+      g_signal_connect (draw_area, "enter-notify-event", G_CALLBACK (label_enter), NULL);
+      g_signal_connect (draw_area, "leave-notify-event", G_CALLBACK (label_leave), NULL);
+      gtk_widget_set_halign (draw_area, GTK_ALIGN_CENTER);
+      gtk_widget_set_valign (draw_area, GTK_ALIGN_CENTER);
+      gtk_box_pack_start (GTK_BOX (cpuFreq->box), draw_area, TRUE, TRUE, 0);
+      cpuFreq->label.draw_area = draw_area;
+    }
+  }
+  else
+  {
+    if (cpuFreq->label.draw_area)
+    {
+      gtk_widget_destroy (cpuFreq->label.draw_area);
+      cpuFreq->label.draw_area = NULL;
+    }
+
+    if (cpuFreq->label.text)
+    {
+      g_free (cpuFreq->label.text);
+      cpuFreq->label.text = NULL;
+    }
   }
 }
 
@@ -733,16 +972,16 @@ cpufreq_widgets (void)
   gtk_container_set_border_width (GTK_CONTAINER (cpuFreq->box), BORDER);
   gtk_container_add (GTK_CONTAINER (cpuFreq->button), cpuFreq->box);
 
-  cpufreq_update_icon (cpuFreq);
+  cpufreq_update_icon ();
 
-  cpufreq_prepare_label (cpuFreq);
+  cpufreq_prepare_label ();
 
   g_signal_connect (cpuFreq->button, "button-press-event",
                     G_CALLBACK (cpufreq_overview), cpuFreq);
 
   /* activate panel widget tooltip */
   g_object_set (G_OBJECT (cpuFreq->button), "has-tooltip", TRUE, NULL);
-  g_signal_connect (G_OBJECT (cpuFreq->button), "query-tooltip",
+  g_signal_connect (cpuFreq->button, "query-tooltip",
                     G_CALLBACK (cpufreq_update_tooltip), cpuFreq);
 
   gtk_widget_show_all (cpuFreq->button);
@@ -781,6 +1020,7 @@ cpufreq_read_config (void)
   options->show_warning        = xfce_rc_read_bool_entry (rc, "show_warning", TRUE);
   options->keep_compact        = xfce_rc_read_bool_entry (rc, "keep_compact", FALSE);
   options->one_line            = xfce_rc_read_bool_entry (rc, "one_line", FALSE);
+  options->icon_color_freq     = xfce_rc_read_bool_entry (rc, "icon_color_freq", FALSE);
   options->unit                = xfce_rc_read_int_entry  (rc, "freq_unit", UNIT_DEFAULT);
 
   if (!options->show_label_freq && !options->show_label_governor)
@@ -798,10 +1038,7 @@ cpufreq_read_config (void)
 
   value = xfce_rc_read_entry (rc, "fontname", NULL);
   if (value)
-  {
-    g_free (options->fontname);
-    options->fontname = g_strdup (value);
-  }
+    cpufreq_set_font (value);
 
   value = xfce_rc_read_entry (rc, "fontcolor", NULL);
   if (value)
@@ -838,12 +1075,18 @@ cpufreq_write_config (XfcePanelPlugin *plugin)
   xfce_rc_write_bool_entry (rc, "show_warning",        options->show_warning);
   xfce_rc_write_bool_entry (rc, "keep_compact",        options->keep_compact);
   xfce_rc_write_bool_entry (rc, "one_line",            options->one_line);
+  xfce_rc_write_bool_entry (rc, "icon_color_freq",     options->icon_color_freq);
   xfce_rc_write_int_entry  (rc, "freq_unit",           options->unit);
 
   if (options->fontname)
-    xfce_rc_write_entry  (rc, "fontname",            options->fontname);
+    xfce_rc_write_entry (rc, "fontname", options->fontname);
+  else
+    xfce_rc_delete_entry (rc, "fontname", FALSE);
+
   if (options->fontcolor)
-    xfce_rc_write_entry  (rc, "fontcolor",           options->fontcolor);
+    xfce_rc_write_entry (rc, "fontcolor", options->fontcolor);
+  else
+    xfce_rc_delete_entry (rc, "fontcolor", FALSE);
 
   xfce_rc_close (rc);
 }
@@ -882,9 +1125,22 @@ cpufreq_free (XfcePanelPlugin *plugin)
 
   g_ptr_array_free (cpuFreq->cpus, TRUE);
 
+  g_free (cpuFreq->cpu_avg);
+  g_free (cpuFreq->cpu_max);
+  g_free (cpuFreq->cpu_min);
+
+  cpufreq_destroy_icons ();
+
+  if (cpuFreq->label.font_desc)
+    pango_font_description_free (cpuFreq->label.font_desc);
+  g_free (cpuFreq->label.text);
+
   g_free (cpuFreq->options->fontname);
+  g_free (cpuFreq->options->fontcolor);
+  g_free (cpuFreq->options);
   cpuFreq->plugin = NULL;
   g_free (cpuFreq);
+  cpuFreq = NULL;
 }
 
 
@@ -895,15 +1151,14 @@ cpufreq_set_size (XfcePanelPlugin *plugin, gint size, CpuFreqPlugin *cpufreq)
   cpuFreq->panel_size = size;
   cpuFreq->panel_rows = xfce_panel_plugin_get_nrows (plugin);
 
-  cpufreq_update_icon (cpufreq);
+  cpufreq_update_icon ();
   cpufreq_update_plugin (TRUE);
 
   return TRUE;
 }
 
 static void
-cpufreq_show_about(XfcePanelPlugin *plugin,
-           CpuFreqPlugin *cpufreq)
+cpufreq_show_about(XfcePanelPlugin *plugin, CpuFreqPlugin *cpufreq)
 {
   const gchar *auth[] = {
     "Thomas Schreck <shrek@xfce.org>",
@@ -941,14 +1196,14 @@ cpufreq_construct (XfcePanelPlugin *plugin)
   cpuFreq->panel_mode = xfce_panel_plugin_get_mode (cpuFreq->plugin);
   cpuFreq->panel_rows = xfce_panel_plugin_get_nrows (cpuFreq->plugin);
   cpuFreq->panel_size = xfce_panel_plugin_get_size (cpuFreq->plugin);
-  cpuFreq->label_max_width = -1;
   cpuFreq->cpus = g_ptr_array_new ();
 
   cpufreq_read_config ();
+  cpuFreq->label.reset_size = TRUE;
   cpuFreq->layout_changed = TRUE;
 
 #ifdef __linux__
-  if (cpufreq_linux_init () == FALSE)
+  if (!cpufreq_linux_init ())
     xfce_dialog_show_error (NULL, NULL,
       _("Your system is not configured correctly to support CPU frequency scaling!"));
 
@@ -970,7 +1225,7 @@ cpufreq_construct (XfcePanelPlugin *plugin)
   xfce_panel_plugin_menu_show_configure (plugin);
   g_signal_connect (plugin, "configure-plugin", G_CALLBACK (cpufreq_configure), NULL);
   xfce_panel_plugin_menu_show_about(plugin);
-  g_signal_connect (G_OBJECT (plugin), "about", G_CALLBACK (cpufreq_show_about), cpuFreq);
+  g_signal_connect (plugin, "about", G_CALLBACK (cpufreq_show_about), cpuFreq);
 }
 
 XFCE_PANEL_PLUGIN_REGISTER (cpufreq_construct);
