@@ -32,6 +32,10 @@
 #define CPU_MAX (-3)
 #define CPU_DEFAULT CPU_MAX
 
+#define FREQ_HIST_BINS 128           /* number of bins */
+#define FREQ_HIST_MAX  (8*1000*1000) /* frequency in kHz */
+#define FREQ_HIST_MIN  0             /* frequency in kHz */
+
 typedef enum
 {
   UNIT_AUTO,
@@ -44,13 +48,14 @@ typedef enum
 typedef struct
 {
   guint  cur_freq;  /* frequency in kHz */
-  guint  max_freq;
+  guint  max_freq_measured;
+  guint  max_freq_nominal;
   guint  min_freq;
   gchar  *cur_governor;
   gchar  *scaling_driver;
 
-  GList* available_freqs;
-  GList* available_governors;
+  GList *available_freqs;
+  GList *available_governors;
 
   gboolean online;
 } CpuInfo;
@@ -66,12 +71,13 @@ typedef struct
 {
   guint       timeout;       /* time between refresh */
   gint        show_cpu;      /* cpu number in panel, or CPU_MIN/AVG/MAX */
-  gboolean    show_icon;
-  gboolean    show_label_governor;
-  gboolean    show_label_freq;
-  gboolean    show_warning;
-  gboolean    keep_compact;
-  gboolean    one_line;
+  gboolean    show_icon:1;
+  gboolean    show_label_governor:1;
+  gboolean    show_label_freq:1;
+  gboolean    show_warning:1;
+  gboolean    keep_compact:1;
+  gboolean    one_line:1;
+  gboolean    icon_color_freq:1;
   gchar      *fontname;
   gchar      *fontcolor;
   CpuFreqUnit unit;
@@ -96,10 +102,25 @@ typedef struct
   IntelPState *intel_pstate;
 
   /* Widgets */
-  GtkWidget *button, *box, *icon, *label_orNull;
-  GtkCssProvider *label_css_provider;
+  GtkWidget *button, *box, *icon;
+  struct {
+    GtkWidget            *draw_area;
+    PangoFontDescription *font_desc;
+    gboolean              reset_size;
+    gchar                *text;
+  } label;
   gboolean layout_changed;
-  gint label_max_width;
+
+  GdkPixbuf *base_icon;
+  GdkPixbuf *current_icon_pixmap;
+  GdkPixbuf *icon_pixmaps[32];  /* table with frequency color coded pixbufs */
+
+  /* Histogram of measured frequencies:
+   *  min: FREQ_HIST_MIN
+   *  max: FREQ_HIST_MAX
+   *  range: max - min
+   *  resolution: range / FREQ_HIST_BINS = 62.5 MHz */
+  guint16 freq_hist[FREQ_HIST_BINS];
 
   CpuFreqPluginOptions *options;
   gint timeoutHandle;
@@ -110,13 +131,16 @@ extern CpuFreqPlugin *cpuFreq;
 G_BEGIN_DECLS
 
 void
-cpufreq_prepare_label (CpuFreqPlugin *cpufreq);
+cpufreq_prepare_label (void);
 
 void
 cpufreq_restart_timeout (void);
 
 void
-cpufreq_update_icon (CpuFreqPlugin *cpufreq);
+cpufreq_set_font (const gchar *fontname_or_null);
+
+void
+cpufreq_update_icon (void);
 
 gboolean
 cpufreq_update_plugin (gboolean reset_label_size);
